"""Some functions around paths. You can find here `find_path()` to find some path efficiently in
some folder, excluding some other inner folders (like venv, node_modules etc.). There is also function
to get desktop path in posix way.
"""

from mypythontools.paths.paths_internal import (
    find_path,
    get_desktop_path,
    isFolderEmpty,
    is_path_free,
    validate_path,
    PathLike,
)

__all__ = ["find_path", "get_desktop_path", "isFolderEmpty", "is_path_free", "validate_path", "PathLike"]



"""Add some stuff to python subprocess, sys, os and platform libraries.

You can find here formatting errors, resolving venv script paths, finding script path, unifying syntax for
linux and windows etc. Most used function is 'terminal_do_command' comunicating with system shell. 
Use strings as input and if there can be space in some command, use 'get_console_str_with_quotes'.
"""
from mypythontools.system.system_internal import (
    check_library_is_available,
    check_script_is_available,
    get_console_str_with_quotes,
    is_wsl,
    PYTHON,
    SHELL_AND,
    terminal_do_command,
    which,
)

__all__ = [
    "check_library_is_available",
    "check_script_is_available",
    "get_console_str_with_quotes",
    "is_wsl",
    "PYTHON",
    "SHELL_AND",
    "terminal_do_command",
    "which",
]



"""Some tools/functions/snippets/files used across projects.

.. image:: https://img.shields.io/pypi/pyversions/mypythontools.svg
    :target: https://pypi.python.org/pypi/mypythontools/
    :alt: Python versions

.. image:: https://badge.fury.io/py/mypythontools.svg
    :target: https://badge.fury.io/py/mypythontools
    :alt: PyPI version

.. image:: https://pepy.tech/badge/mypythontools
    :target: https://pepy.tech/project/mypythontools
    :alt: Downloads

.. image:: https://img.shields.io/lgtm/grade/python/github/Malachov/mypythontools.svg
    :target: https://lgtm.com/projects/g/Malachov/mypythontools/context:python
    :alt: Language grade: Python

.. image:: https://readthedocs.org/projects/mypythontools/badge/?version=latest
    :target: https://mypythontools.readthedocs.io/en/latest/?badge=latest
    :alt: Documentation Status

.. image:: https://img.shields.io/badge/License-MIT-yellow.svg
    :target: https://opensource.org/licenses/MIT
    :alt: License: MIT

.. image:: https://codecov.io/gh/Malachov/mypythontools/branch/master/graph/badge.svg
    :target: https://codecov.io/gh/Malachov/mypythontools
    :alt: Codecov


It's called mypythontools, but it's also made for you...

Many projects - one codebase.

There is also some extra stuff, that is not bundled via PyPI (CSS for readthedocs etc.),
such a content is under the `Tools` topic.


Links
=====

Official documentation - https://mypythontools.readthedocs.io/

Official repo - https://github.com/Malachov/mypythontools

Installation
============

Python >=3.6 (Python 2 is not supported).

Install with::

    pip install mypythontools


Python library
==============

**subpackages**

- :py:mod:`mypythontools.config`
- :py:mod:`mypythontools.misc`
- :py:mod:`mypythontools.paths`
- :py:mod:`mypythontools.plots`
- :py:mod:`mypythontools.property`
- :py:mod:`mypythontools.system`
- :py:mod:`mypythontools.terminal`
- :py:mod:`mypythontools.types`

Subpackages names are self describing and you can find documentation in subpackages docstrings.

Mypythontools_cicd
==================

There is extra library in separate repository

https://github.com/Malachov/mypythontools_cicd

This can help you with a lot of stuff around CICD like getting project paths, generating docs, testing,
deploying to PyPi etc.

Tools
=====

There are some extra tools not included in in python library (installable via pip), but still on GitHub
repository. Some tools were big enough to be refactored to own repository. Still listed here though. 

Link where you can find that content:

https://github.com/Malachov/mypythontools/tree/master/tools

Link where you can read about how to use it:

https://mypythontools.readthedocs.io/#Tools

Some examples of what you can find only on GitHub

requirements
------------

Install many libraries at once (no need for Anaconda). Download `requirements.txt` file from
https://github.com/Malachov/mypythontools/tree/master/tools/requirements and in that folder use::

    pip install -r requirements.txt

It's good for python libraries that other users with different versions of libraries will use. If not
standalone application where freezing into virtual env is good idea - here is possible to use these
requirements with using --upgrade from time to time to be sure that your library will be working for
up-to-date version of dependencies.

sphinx-alabaster-css
--------------------

Its good idea to generate documentation from code. If you are using sphinx and alabaster theme, you can use
this css file for formatting.

Tested on readthedocs hosting (recommended).

CSS are served from GitHub, and it's possible to change on one place and edit how all projects docs look like
at once.

Just add this to sphinx conf.py::

>>> html_css_files = ["https://malachov.github.io/readthedocs-sphinx-alabaster-css/custom.css"]

Also, of course if you want you can download it and use locally from project if you need.

Result should look like this

.. image:: /_static/sphinx-alabaster-css.png
    :width: 620
    :alt: sphinx-alabaster-css
    :align: center
"""
from mypythontools import config, misc, paths, plots, property, system, types

__all__ = ["config", "misc", "paths", "plots", "property", "system", "types"]

__version__ = "2.0.0"

__author__ = "Daniel Malachov"
__license__ = "MIT"
__email__ = "malachovd@seznam.cz"



"""Module with functions for 'misc' subpackage."""

from __future__ import annotations
from typing import Callable, Any, Iterable
import builtins
import time
import sys
from pathlib import Path
import os
import shutil

from typing_extensions import Literal
import pandas as pd
from tabulate import tabulate

from ..paths import PathLike, validate_path


class Emojis:
    """Emojis that can be printed."""

    PARTY = "🎉"  # "\U0001F389"
    DISAPPOINTMENT = " ☹️ "  # "\U0001F641"


EMOJIS = Emojis()


class GlobalVars:
    """Global variables that can be useful."""

    @property
    def jupyter(self):
        """If runs in Jupyter, it returns True."""
        return True if hasattr(builtins, "__IPYTHON__") else False

    @property
    def is_tested(self):
        """If is tested with Pytest, it returns True."""
        return True if "PYTEST_CURRENT_TEST" in os.environ else False


GLOBAL_VARS = GlobalVars()

DEFAULT_TABLE_FORMAT = {
    "tablefmt": "grid",
    "floatfmt": ".3f",
    "numalign": "center",
    "stralign": "center",
}


class TimeTable:
    """Class that create printable table with spent time on various phases that runs sequentionally.

    Add entry when current phase end (not when it starts).

    Example:
        >>> import time
        ...
        >>> time_table = TimeTable()
        >>> time.sleep(0.01)
        >>> time_table.add_entry("First phase")
        >>> time.sleep(0.02)
        >>> time_table.add_entry("Second phase")
        >>> time_table.add_entry("Third phase")
        >>> time_table.finish_table()
        ...
        >>> print(time_table.time_table)
        +--------------+--------------+
        |     Time     |  Phase name  |
        +==============+==============+
        | First phase  |    0...
    """

    def __init__(self) -> None:
        """Init the table."""
        self.time_df: pd.DataFrame = pd.DataFrame()
        self.time_table: str = ""
        self.times: list[tuple[str, float]] = []
        self.last_time: float = time.time()

    def add_entry(self, phase_name: str) -> None:
        """Add new line to the Time table."""
        self.times.append((phase_name, round((time.time() - self.last_time), 3)))

    def finish_table(self, table_format: None | dict = None) -> None:
        """Create time table.

        Args:
            table_format (None | dict, optional): Dict of format settings used in tabulate. If None, default
                DEFAULT_TABLE_FORMAT is used. Defaults to None.
        """
        if not table_format:
            table_format = DEFAULT_TABLE_FORMAT

        self.add_entry("Completed")
        self.time_df = pd.DataFrame(self.times, columns=["Time", "Phase name"])
        self.time_table = tabulate(self.time_df.values, headers=list(self.time_df.columns), **table_format)


def watchdog(timeout: int | float, function: Callable, *args, **kwargs) -> Any:
    """Time-limited execution for python function. TimeoutError raised if not finished during defined time.

    Args:
        timeout (int | float): Max time execution in seconds.
        function (Callable): Function that will be evaluated.
        *args: Args for the function.
        *kwargs: Kwargs for the function.

    Raises:
        TimeoutError: If defined time runs out.
        RuntimeError: If function call with defined params fails.

    Returns:
        Any: Depends on used function.

    Examples:
        >>> import time
        >>> def sleep(sec):
        ...     for _ in range(sec):
        ...         time.sleep(1)
        >>> watchdog(1, sleep, 0)
        >>> watchdog(1, sleep, 10)
        Traceback (most recent call last):
        TimeoutError: ...
    """
    old_tracer = sys.gettrace()

    def tracer(frame, event, arg, start=time.time()):
        """Sys trace helpers that checks the time for watchdog."""
        now = time.time()
        if now > start + timeout:
            raise TimeoutError("Time exceeded")
        return tracer if event == "call" else None

    try:
        sys.settrace(tracer)
        result = function(*args, **kwargs)

    except TimeoutError:
        sys.settrace(old_tracer)
        raise TimeoutError(
            "Timeout defined in watchdog exceeded.",
        )

    except Exception as err:
        sys.settrace(old_tracer)
        raise RuntimeError(
            f"Watchdog with function {function.__name__}, args {args} and kwargs {kwargs} failed."
        ) from err

    finally:
        sys.settrace(old_tracer)

    return result


def delete_files(paths: PathLike | Iterable[PathLike], on_error: Literal["pass", "raise"] = "pass"):
    """Delete file, folder or Sequence of files or folders.

    Folder can contain files, it will be also recursively deleted. You can choose behavior on error (because
    of permissions for example).

    Args:
        paths (PathLike | Iterable[PathLike]): List or tuple of paths to be deleted. Can be files as well
            as folders.
        on_error (Literal["pass", "raise"], optional): Depends whether you want to pass or raise an error.
            Error can occur when for example file is opened or if has no necessary permissions.
            Defaults to "pass".
    """
    if isinstance(paths, (Path, str, os.PathLike)):
        paths = [paths]

    for i in paths:
        try:
            delete_path = Path(i)

            if delete_path.exists():
                if delete_path.is_dir():
                    shutil.rmtree(delete_path, ignore_errors=False)
                else:
                    delete_path.unlink()

        except (FileNotFoundError, OSError):
            if on_error == "raise":
                raise


def print_progress(name: str, verbose: bool = True):
    """Print current step of some process.

    Divide it with newlines so it's more readable.

    Args:
        name (str): Name current step.
        verbose (bool): It is possible to turn off logging of progress with one parameter config value.
            Defaults to True.
    """
    if verbose:
        print(f"\n{name}\n")



"""Module with functions for 'terminal' subpackage."""

from __future__ import annotations
from pathlib import Path
import subprocess
import platform
import importlib.util
import sys

from ..paths import PathLike


def is_wsl():
    """Check whether script runs in Windows Subsystem for Linux or not.

    Returns:
        bool: True or False based on using wsl.
    """
    return "microsoft-standard" in platform.uname().release


SHELL_AND = " && "
"""If using some nonstandard shell can be edited here globally. ';' can be used if && not working."""
PYTHON = "python" if platform.system() == "Windows" and not is_wsl() else "python3"


class TerminalCommandError(Exception):
    """Error that function `terminal_do_command` raises.

    Just changing the name of exception, so it's clear where it come from.
    """

    pass


def terminal_do_command(
    command: str,
    shell: bool = True,
    cwd: None | PathLike = None,
    verbose: bool = True,
    error_header: str = "",
):
    """Run command in terminall and process output.

    Args:
        command (str): Command to run.
        shell (bool, optional): Same meaning as in ``subprocess.run()``. Defaults to False.
        cwd (None | PathLike, optional): Same meaning as in ``subprocess.run()``. Defaults to None.
        verbose (bool, optional): Whether print output to console. Defaults to True.
        error_header (str, optional): If meet error, message at the beginning of message. Default to "".

    Raises:
        RuntimeError: When process fails to finish or return non zero return code.
    """
    error = None
    context = None

    try:
        result = subprocess.run(command, shell=shell, cwd=cwd, capture_output=True)
        if result.returncode == 0:
            if verbose:
                striped = "\r\n"
                print(f"\n{result.stdout.decode().strip(striped)}\n")
        else:
            stderr = result.stderr.decode().strip("\r\n")
            stdout = result.stdout.decode().strip("\r\n")
            error = f"\n\nstderr:\n\n{stderr}\n\nstdout:\n\n{stdout}\n\n"

    except Exception as err:  # pylint: disable=broad-except
        error = "Suprocess command crashed internally in subprocess and did not finished."
        context = err

    if error:
        header = f"{error_header}\n\n" if error_header else ""
        cwd_str = "on your project root" if cwd is None else f"in '{cwd}' folder"

        raise TerminalCommandError(
            f"{header}"
            f"Running command in terminal failed. Try command below in the terminal {cwd_str} "
            f"\n\n{command}\n\n"
            "On windows use cmd so script paths resolved correctly. Try it with administrator rights in\n"
            "your project root folder. Permission error may be one of the typical issue or some\n"
            "necessary library missing or installed elsewhere than in used venv.\n\n"
            f"Captured error: {error}",
        ) from context


def get_console_str_with_quotes(string: PathLike):
    """In terminal if value or contain spaces, it's not taken as one param.

    This wraps it with quotes to be able to use paths and values as needed. Alternative to this function is to
    use python shlex library, list of commands and 'shlex.join' to get the command string.

    Args:
        string (str, Path): String  to be edited.

    Returns:
        str: Wrapped string that can be used in terminal.

    Example:
        >>> get_console_str_with_quotes("/path to file/file")
        '"/path to file/file"'
    """
    if isinstance(string, (Path)):
        string = string.as_posix()
    if not isinstance(string, str):
        string = str(string)
    string = string.strip("'")
    string = string.strip('"')
    return f'"{string}"'


def which(name):
    """Return path of defined script.

    It's similar to `whichcraft.which` or `shutil.which` with the difference that it will work also if calling
    with `wsl python code_with_which.py`. If script not available, None is returned. If it's found, pathlib
    Path is returned, not string.

    Args:
        name (str): Name of the script. For example `python`, 'black', or 'pytest'.

    Returns:
        None | Path: Path to script or None if it's not available.

    Example:
        >>> which("black").exists()
        True
    """
    if is_wsl():
        which = f". {sys.prefix}/bin/activate && which "
    else:
        which = "where " if platform.system() == "Windows" else "which "

    command = which + name

    result = subprocess.run(command, shell=True, capture_output=True)

    output = result.stdout.decode().rstrip("\r\n")

    if "\r\n" in output:
        output = output.split("\r\n")[0]

    if result.returncode != 0 or not output or not Path(output).exists():
        return None
    else:
        return Path(output)


def check_script_is_available(name, install_library=None, message="default"):
    """Check if python script is available.

    This doesn't need to be installed in current venv, but anywhere on computer.

    Args:
        name (str): Name of the script. E.g "black'.
        install_library (str, optional): Install script with this library added to default message.
            Defaults to None.
        message (str, optional): Message that will be printed when library not installed.
            Defaults to "default".

    Raises:
        RuntimeError: If module is installed, error is raised.

    Example:
        >>> check_script_is_available("black")
        >>> check_script_is_available("not_existing_script")
        Traceback (most recent call last):
        RuntimeError: ...
    """
    if message == "default":
        message = f"Python script {name} is necessary and not available. "

    if install_library:
        message = message + f"\nTo get this executable available, do \n\n\tpip install {name}\n\n"

    if not which(name):
        raise RuntimeError(message)


def check_library_is_available(name, message="default", extras: str | None = None):
    """Make one-liner for checking whether some library is installed.

    If running on venv, it checks only this venv, no global site packages.

    Args:
        name (str): Name of the library.
        message (str, optional): Message that will be printed when library not installed.
            Defaults to "default".
        extras (str, optional): Sometimes there are sections of requirements that are installable with extra
            pip command e.g. `pip install mypythontools[plots]`. If you define a name of a library with name
            of extras, for example 'mypythontools[plots]', it will be printed to the user, so he is able to
            install all libraries at once. Defaults to None.

    Raises:
        ModuleNotFoundError: If module is installed, error is raised.

    Example:
        >>> check_library_is_available("typing_extensions")
        >>> check_library_is_available("not_installed_lib")
        Traceback (most recent call last):
        ModuleNotFoundError: ...
    """
    if message == "default":
        message = (
            f"Library {name} is necessary and not available. Some libraries are used in just for small"
            f"part of module, so not installed by default. Use \n\n\tpip install {name}\n\n"
        )

    if extras:
        message = message + (
            "There may be more libraries you will need for this use case. There are extra requirements to be "
            "installed. You can install it with  \n\n\tpip install {extras}\n\n"
        )

    if not importlib.util.find_spec(name):
        raise ModuleNotFoundError(message)



"""Runs before every pytest test. Used automatically (at least at VS Code)."""
from __future__ import annotations

import mypythontools_cicd as cicd

cicd.tests.setup_tests(matplotlib_test_backend=True)



"""Module with functions for 'property' subpackage."""

from __future__ import annotations

from typing import Generic, TypeVar, Callable, Type, overload, Any

from typeguard import check_type

from .. import types


T = TypeVar("T")
U = TypeVar("U")


# Needs to inherit from property to be able to use help tooltip
class MyPropertyClass(property, Generic[T]):
    """Python property on steroids. Check module docstrings for more info."""

    # Property is inherited just for formatting help in IDE, so not called from init
    def __init__(self, fget: Callable[..., T], doc=None):  # pylint: disable=super-init-not-called
        """Init property."""
        if fget:
            self.allowed_types = types.get_return_type_hints(fget)

            self.init_function = fget

            if isinstance(fget, staticmethod):
                inner_func = fget.__func__
            else:
                inner_func = fget

            if doc:
                self.__doc__ = doc
            elif inner_func.__doc__:
                self.__doc__ = fget.__doc__
            else:
                self.__doc__ = None

        self.public_name = ""
        self.private_name = ""

    def default_fset(self, used_object, content) -> None:
        """Define how new values will be stored."""
        setattr(used_object, self.private_name, content)

    def __set_name__(self, _, name):
        """Define names. Private usually with underscore."""
        self.public_name = name
        self.private_name = "_" + name

    @overload
    def __get__(self, used_object: None, objtype: Any = None) -> MyPropertyClass[T]:
        ...

    @overload
    def __get__(self, used_object: U, objtype: Type[U] = None) -> T:
        ...

    def __get__(self, used_object, objtype=None):
        """Define what happens when you want access config attribute.

        If used on class, class itself is returned.
        """
        if not used_object:
            return self

        # Expected value can be nominal value or function, that return that value
        content = getattr(used_object, self.private_name)
        if isinstance(content, staticmethod):
            content = content.__func__

        if not hasattr(content, "myproperties_list") and callable(content):
            # Depends whether it's staticmethod or not
            try:
                value = content(used_object)
            except TypeError:
                value = content()

        else:
            value = content

        return value

    def __set__(self, used_object, content: T | Callable[..., T]):
        """Define what happen if user set new config value."""
        # You can setup value or function, that return that value
        if not hasattr(content, "myproperties_list") and callable(content):
            result = content(used_object)
        else:
            result = content

        if self.allowed_types:
            check_type(expected_type=self.allowed_types, value=result, argname=self.public_name)

        self.default_fset(used_object, result)


# Define as static method as it can be used directly
def init_my_properties(self):
    """Init private values of properties.

    Property usually get value of some private variable. This leads to many unnecessary code sometimes.
    """
    if not hasattr(self, "myproperties_list"):
        setattr(self, "myproperties_list", [])

    for i in vars(type(self)).values():
        if isinstance(i, MyPropertyClass):
            self.myproperties_list.append(i.public_name)
            setattr(
                self,
                i.private_name,
                i.init_function,
            )


def MyProperty(f: Callable[..., T]) -> MyPropertyClass[T]:  # pylint: disable=invalid-name
    """Wrap MyProperty to work as expected.

    If not using this workaround, but use class decorator, IDE complains that property has no defined
    setter.

    Args:
        f(Callable[..., T]): Usually it's decorated method from class. It can be static as well.
    """
    return MyPropertyClass[T](f)


# TODO - Use PEP 614 and define type just i n class decorator
# Python 3.9 necessary



"""Module with functions for 'paths' subpackage."""

from __future__ import annotations
from typing import Sequence, Union
from pathlib import Path

from ..types import validate_sequence

PathLike = Union[Path, str]  # Path is included in PathLike
"""Str pr pathlib Path. It can be also relative to current working directory."""


def find_path(
    name: str,
    folder: PathLike | None = None,
    exclude_names: Sequence[str] = ("node_modules", "build", "dist"),
    exclude_paths: Sequence[PathLike] = (),
    levels: int = 5,
) -> Path:
    """Search for file or folder in defined folder (cwd() by default) and return it's path.

    Args:
        name (str): Name of folder or file that should be found. If using file, use it with extension
            e.g. "app.py".
        folder (PathLike | None, optional): Where to search. If None, then root is used (cwd by default).
            Defaults to None.
        exclude_names (Sequence[str], optional): List or tuple of ignored names. If this name is whenever in
            path, it will be ignored. Defaults to ('node_modules', 'build', 'dist').
        exclude_paths (Sequence[PathLike], optional): List or tuple of ignored paths. If defined path is
            subpath of found file, it will be ignored. If relative, it has to be from cwd. Defaults to ().
        levels (str, optional): Recursive number of analyzed folders. Defaults to 5.

    Returns:
        Path: Found path.

    Raises:
        FileNotFoundError: If file is not found.
    """
    validate_sequence(exclude_names, "exclude_names")
    validate_sequence(exclude_paths, "exclude_paths")

    folder = Path.cwd() if not folder else validate_path(folder)

    for lev in range(levels):
        glob_file_str = f"{'*/' * lev}{name}"

        for i in folder.glob(glob_file_str):
            is_wanted_file = True
            for j in exclude_names:
                if j in i.parts:
                    is_wanted_file = False
                    break

            if is_wanted_file:
                for j in exclude_paths:
                    excluded_name = Path(j).resolve()
                    if i.as_posix().startswith(excluded_name.as_posix()):
                        is_wanted_file = False
                        break

            if is_wanted_file:
                return i

    # If not returned - not found
    raise FileNotFoundError(f"File `{name}` not found")


def get_desktop_path() -> Path:
    """Get desktop path.

    Returns:
        Path: Return pathlib Path object. If you want string, use `.as_posix()`

    Example:
        >>> desktop_path = get_desktop_path()
        >>> desktop_path.exists()
        True
    """
    return Path.home() / "Desktop"


def validate_path(path: PathLike) -> Path:
    """Convert to pathlib path, resolve to full path and check if exists.

    Args:
        path (PathLike): Validated path.

    Raises:
        FileNotFoundError: If file do not exists.

    Returns:
        Path: Pathlib Path object.

    Example:
        >>> from pathlib import Path
        >>> existing_path = validate_path(Path.cwd())
        >>> non_existing_path = validate_path("not_existing")
        Traceback (most recent call last):
        FileNotFoundError: ...
    """
    path = Path(path).resolve()
    if not path.exists():
        raise FileNotFoundError(f"File nor folder found on defined path {path}")
    return path


def isFolderEmpty(path: PathLike) -> bool:
    """Check whether folder is empty.

    Args:
        path (PathLike): Path to folder

    Raises:
        RuntimeError: If there is no folder on path.

    Returns:
        bool: True or False

    Example:
        >>> from pathlib import Path
        >>> from shutil import rmtree
        >>> test_path = Path("isFolderEmptyFolder")
        >>> test_path.mkdir()
        >>> isFolderEmpty(test_path)
        True
        >>> isFolderEmpty(test_path.parent)
        False
        >>> rmtree("isFolderEmptyFolder")
    """
    path = validate_path(path)

    if not path.is_dir():
        raise RuntimeError("On defined path is not folder.")

    content = next(path.iterdir(), None)

    if content is None:
        return True
    else:
        return False


def is_path_free(path: PathLike):
    """Check whether path is available. It means that it doesn't exists yet or its a folder, but it's empty.

    Args:
        path (PathLike): Path to be verified.

    Returns:
        bool: True or False

    Example:
        >>> from pathlib import Path
        >>> from shutil import rmtree
        >>> is_path_free("non/existing/path")
        True
        >>> test_path = Path("isFolderEmptyFolder")
        >>> test_path.mkdir()
        >>> is_path_free(test_path)
        True
        >>> is_path_free(test_path.parent)
        False
        >>> rmtree("isFolderEmptyFolder")
    """
    path = Path(path)
    if not Path(path).exists():
        return True
    else:
        if path.is_dir:
            if isFolderEmpty(path):
                return True
    return False



"""Sphinx config.

Reference: https://www.sphinx-doc.org/en/master/usage/configuration.html
"""

from __future__ import annotations
import sys
import pathlib
import datetime


# Suppose separate build and source structure and logo.png in _static folder

# Settings
PROJECT = "mypythontools"
AUTHOR = "Daniel Malachov"
GITHUB_USER = "Malachov"

# End of settings
###################

# Folders to sys path to be able to import
script_dir = pathlib.Path(__file__).resolve()
root = script_dir.parents[2]
lib_path = root / PROJECT

for i in [script_dir, root, lib_path]:
    if i.as_posix() not in sys.path:
        sys.path.insert(0, i.as_posix())


# -- Project information -----------------------------------------------------

copyright = f"2020, {AUTHOR}"  # pylint: disable=redefined-builtin

# The full version, including alpha/beta/rc tags
release = datetime.datetime.now().strftime("%d-%m-%Y")

master_doc = PROJECT  # pylint: disable=invalid-name

source_suffix = [".rst", ".md"]

# -- General configuration ---------------------------------------------------
html_theme_options = {
    "github_user": GITHUB_USER,
    "github_repo": PROJECT,
    "github_banner": True,
}


# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.napoleon",
    "sphinx.ext.viewcode",
    "sphinx.ext.githubpages",
    "sphinx.ext.autosectionlabel",
    # "sphinx.ext.intersphinx",
    # "sphinx.ext.imgmath",
    # "m2r2",
    # "myst_parser",
]

# 'about.html',
html_sidebars = {"**": ["navi.html", "searchbox.html"]}

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "alabaster"  # pylint: disable=invalid-name

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# html_extra_path = ['../extra']

html_css_files = [
    "https://malachov.github.io/mypythontools/tools/sphinx-alabaster-css/custom.css",
]

napoleon_custom_sections = [
    ("Types", "returns_style"),
    ("Type", "returns_style"),
    ("Options", "returns_style"),
    ("Default", "returns_style"),
    ("For example", "returns_style"),
]



"""Push the CI pipeline. Format, create commit from all the changes, push and deploy to PyPi."""

from mypythontools_cicd.project_utils import project_utils_pipeline, default_pipeline_config

default_pipeline_config.deploy = True
# default_pipeline_config.do_only = ""


if __name__ == "__main__":

    default_pipeline_config.do_only = "deploy"
    # All the parameters can be overwritten via CLI args
    project_utils_pipeline(config=default_pipeline_config)
